@@ -1,5 +1,5 @@
 /*  BumpRace: an easy-to-learn arcade game for up to two players
-    Copyright (C) 2000 Karl Bartel
+    Copyright (C) 2006 Karl Bartel
 
     This program is free software; you can redistribute it and/or modify        
     it under the terms of the GNU General Public License as published by        
@@ -42,7 +42,7 @@
 #include "options.h"
 
 #ifndef VERSION
-#define VERSION "1.5.1"
+#define VERSION "1.5.3"
 #endif
 #define MAX_PLAYER_NUM 2
 #define FOREGROUND_TILE_NUM 8
@@ -142,11 +142,75 @@ void corner_collision(  Uint8 xblock, Uint8 yblock)
   }
 }
 
+/**
+ * If players p1 and p2 collide, moves them out of the collision and makes them bounce.
+ */
+void player_collision(int p1, int p2) {
+	const float radius = 12; /**< assumed radius of ships */
+
+	// calculate distance between ships
+	float dist_x = user[p1].realx - user[p2].realx;
+	float dist_y = user[p1].realy - user[p2].realy;
+	float dist = sqrt((dist_x * dist_x) + (dist_y * dist_y));
+
+	// abort if ships do not touch/intersect
+	if (dist > (radius + radius)) return;
+
+	// calculate penetration depth
+	float depth = (radius + radius) - dist;
+
+	// calculate unity vector of hit direction
+	float hit_x = dist_x / sqrt((dist_x * dist_x) + (dist_y * dist_y));
+	float hit_y = dist_y / sqrt((dist_x * dist_x) + (dist_y * dist_y));
+
+	// calculate fraction of speeds in direction of hit vector
+	float scalar1 = (user[p1].xspeed * hit_x) + (user[p1].yspeed * hit_y);
+	float scalar2 = (user[p2].xspeed * hit_x) + (user[p2].yspeed * hit_y);
+
+	// abort if nobody seems to have caused the collision
+	if (fabsf(scalar1) + fabsf(scalar2) == 0) return;
+
+	// calculate "contribution" of each ship to the collision as a fraction of 1, judging by its speed
+	// TODO: this penalizes a ship moving *out* of the collision, too
+	float contrib1 = fabsf(scalar1) / (fabsf(scalar1) + fabsf(scalar2));
+	float contrib2 = 1-contrib1;
+
+	// move ships just out of collision with total distance based on "contribution" figure
+	user[p1].realx += hit_x * (depth * contrib1);
+	user[p1].realy += hit_y * (depth * contrib1);
+	user[p2].realx -= hit_x * (depth * contrib2);
+	user[p2].realy -= hit_y * (depth * contrib2);
+
+	// calculate changes in speed
+	float delta1_x = scalar1 * hit_x;
+	float delta1_y = scalar1 * hit_y;
+	float delta2_x = scalar2 * hit_x;
+	float delta2_y = scalar2 * hit_y;
+
+	// modify speeds
+	user[p1].xspeed -= delta1_x;
+	user[p1].yspeed -= delta1_y;
+	user[p1].xspeed += delta2_x;
+	user[p1].yspeed += delta2_y;
+	user[p2].xspeed += delta1_x;
+	user[p2].yspeed += delta1_y;
+	user[p2].xspeed -= delta2_x;
+	user[p2].yspeed -= delta2_y;
+}
+
 void kollision()  //calcualtes all collisions
 {
   Uint8 x,y,x2,y2,sticky;
   SDL_Rect reblit_back;
   if (( keys[user[pl].extra] == SDL_PRESSED )&&(user[pl].sticky_possible)) sticky=1; else sticky=0;
+
+  // handle collisions with other players
+  // TODO: does this belong here?
+  int pl2;
+  for (pl2 = pl+1; pl2 < playernum; pl2++) {
+    player_collision(pl, pl2);
+  }
+
   if (already) {already=0;} else
   for (y=0;y<=14;y++) {
     for (x=0;x<=19;x++) {
@@ -239,21 +303,60 @@ void kollision()  //calcualtes all collisions
   } 
 }
 
-void thrust()
+/**
+ * Add forward thrust to active player's ship.
+ * Set turbo_factor to 1 for normal acceleration
+ */
+void thrust(float turbo_factor)
 {
-  float deltaspeedx, deltaspeedy;
-  
-  deltaspeedy = -cos(user[pl].turn*0.01745)*(game_speed/precision);      // 0.01745=(2*PI)/360 thats RAD to DEG
-  deltaspeedx =  sin(user[pl].turn*0.01745)*(game_speed/precision);
-  user[pl].yspeed+=deltaspeedy;
-  user[pl].xspeed+=deltaspeedx;
-  NextParticle-=game_speed;
-  while (NextParticle<0) {
-      NewParticles(user[pl].turn, user[pl].realx, user[pl].realy);
-      NextParticle+=350;
-  }
+    float deltaspeedx, deltaspeedy;
+	
+	deltaspeedy = -cos(user[pl].turn*0.01745)*turbo_factor*(game_speed/precision);      // 0.01745=(2*PI)/360 thats RAD to DEG
+	deltaspeedx =  sin(user[pl].turn*0.01745)*turbo_factor*(game_speed/precision);
+    user[pl].yspeed+=deltaspeedy;
+    user[pl].xspeed+=deltaspeedx;
+    NextParticle-=game_speed;
+	 
+	float spread_percent;
+	switch( user[pl].racernum ) {
+		case 0: // SlowPoke
+			spread_percent = 100;
+			break;
+		case 1: // Normal
+			spread_percent = 50;
+			break;
+		case 2: // Speedy
+			spread_percent = 10;
+			break;
+		case 3: // Spinny
+			spread_percent = 120;
+			break;
+		default: // OOPS!
+			spread_percent = 0;
+	}
+	if (turbo_factor > 1) spread_percent = 100;
+	
+	while (NextParticle<0) {
+			NewParticles(user[pl].turn, user[pl].realx, user[pl].realy, spread_percent);
+			NextParticle+=350;
+	}
 }
 
+//void thrust()
+//{
+//  float deltaspeedx, deltaspeedy;
+//  
+//  deltaspeedy = -cos(user[pl].turn*0.01745)*(game_speed/precision);      // 0.01745=(2*PI)/360 thats RAD to DEG
+//  deltaspeedx =  sin(user[pl].turn*0.01745)*(game_speed/precision);
+//  user[pl].yspeed+=deltaspeedy;
+//  user[pl].xspeed+=deltaspeedx;
+//  NextParticle-=game_speed;
+//  while (NextParticle<0) {
+//      NewParticles(user[pl].turn, user[pl].realx, user[pl].realy);
+//      NextParticle+=350;
+//  }
+//}
+
 void HandleRacer() //reads keys and sets new coordinates
 {
   float alpha,hyp;
@@ -265,7 +368,7 @@ void HandleRacer() //reads keys and sets new coordinates
     SDL_PollEvent(&event);
     keys = SDL_GetKeyState(NULL);
     if ( keys[user[pl].up] == SDL_PRESSED ) {
-      thrust();
+      thrust(1);
     }
     if ( keys[user[pl].right] == SDL_PRESSED ) {
       user[pl].turn+=turn_speed;
@@ -274,8 +377,7 @@ void HandleRacer() //reads keys and sets new coordinates
       user[pl].turn-=turn_speed;
     }
     if (( keys[user[pl].extra] == SDL_PRESSED )&&(user[pl].turbo_possible)) {
-      user[pl].yspeed-=cos(user[pl].turn*0.01745)*turbo*(game_speed/precision);       
-      user[pl].xspeed+=sin(user[pl].turn*0.01745)*turbo*(game_speed/precision);
+      thrust(turbo);
     }
     user[pl].realx=user[pl].realx+user[pl].xspeed/accel_speed;
     user[pl].realy=user[pl].realy+user[pl].yspeed/accel_speed;
@@ -945,11 +1047,11 @@ void InitLevel()
 //  printf(" successfully **\n");
 }
 
-void StartText()   //bl means brightness loss
+void StartText()
 {
   SDL_Rect rect;
   
-  sprintf(text,"BumpRace Version %s (C) 2000 Karl Bartel", VERSION);
+  sprintf(text,"BumpRace Version %s (C) 2007 Karl Bartel", VERSION);
   PutString(Screen,170, 380, text);
   PutString(Screen,170, 402, "BumpRace comes with ABSOLUTELY NO WARRANTY");
   PutString(Screen,170, 424, "This is free software, and you are welcome to");
